import json
from enum import Enum
from pathlib import Path

from osgeo import ogr, osr

from ru_axioma_gis_loadOsm.ui.tools.add_helper.db.dbTools import DbTools


def ogrLineStringToPolygon(ogr_geometry):
    ring=ogr.Geometry(ogr.wkbLinearRing)
    for pt in ogr_geometry.GetPoints():
        ring.AddPoint_2D(pt[0],pt[1])
    geo_polygon=ogr.Geometry(ogr.wkbPolygon)
    geo_polygon.AddGeometry(ring)
    ring=None
    return geo_polygon
class TypeGeometry(Enum):
    PolyLine=0
    Polygon=1
    Point=2
    Text=3
    Vector=4
    Label=5
    Any=6
class Attribute:
    def __init__(self, name,  type_att, length=None, precission=None):
        self.__name=name
        self.__type_att=type_att
        self.__lenght=length
        self.__precission=precission
    @property
    def Name(self):
        return self.__name
    def getOgrFieldDefn(self):
        if self.__type_att=="int" or self.__type_att=="integer":
            return ogr.FieldDefn(self.Name, ogr.OFTInteger)
        if self.__type_att=="int64" or self.__type_att=="integer64":
            return ogr.FieldDefn(self.Name, ogr.OFTInteger64)
        if self.__type_att=="float" or self.__type_att=="double":
            return ogr.FieldDefn(self.Name, ogr.OFTReal)
        if self.__type_att=="str" or self.__type_att=="string":
            ogr_def = ogr.FieldDefn(self.Name, ogr.OFTString)
            ogr_def.SetWidth(self.__lenght)
            return ogr_def
class DbOgr:
    def __init__(self,path_out,driver_name="GPKG"):
        self.__path_ds=path_out
        self.__name_driver=driver_name
    def createSpatial(self,path_db_create=None):
        path_db=path_db_create
        if path_db is None:
            path_db=self.__path_ds
        self.__sqlite_dr = ogr.GetDriverByName(self.__name_driver)
        isCreate=True
        if Path(path_db).exists():
            isCreate=False
        if isCreate:
            if self.__name_driver=="SQLite":
                dsco = ["SPATIALITE=YES"]
                self.__ds=self.__sqlite_dr.CreateDataSource(path_db, options=dsco)
            else:
                self.__ds = self.__sqlite_dr.CreateDataSource(path_db)
        else:
            self.__ds=ogr.Open(path_db,1)
        if self.__ds is None:
            return False
        self.__path=path_db

        return True
    def open(self,path_db,isEditable=True):
        rd_write=0
        if isEditable:
            rd_write=1
        self.__ds=ogr.Open(path_db,rd_write)
    def runSql(self,sql):
        layer_sql=self.__ds.ExecuteSQL(sql)
        return layer_sql
    def getLayer(self,name_layer):
        if self.__ds is None:
            return None
        layer=self.__ds.GetLayerByName(name_layer)
        return layer
    def createLayer(self, name_layer, attribytes, srs_wkt, type_geometry=None):
        self.__name_layer=name_layer
        layer=OgrLayer()
        layer.createLayer(self.__ds,name_layer, attribytes, srs_wkt, type_geometry=None)
        return layer
    def runSql(self,sql):
        layer_sql=self.__ds.ExecuteSQL(sql)
        return layer_sql

    def createLayerOld(self,name_layer,attribytes,srs_wkt,type_geometry=None):
        srs = osr.SpatialReference()
        if srs_wkt is None:
            srs.ImportFromEPSG(4326)
        else:
            srs.ImportFromWkt(srs_wkt)
        ogr_type_geometry = ogr.wkbUnknown
        if type_geometry == TypeGeometry.PolyLine or type_geometry == TypeGeometry.Vector:
            ogr_type_geometry = ogr.wkbLineString
        if type_geometry == TypeGeometry.Any:
            ogr_type_geometry = ogr.wkbUnknown
        if type_geometry == TypeGeometry.Polygon:
            ogr_type_geometry = ogr.wkbPolygon
        if type_geometry == TypeGeometry.Point:
            ogr_type_geometry = ogr.wkbPoint
        layer = self.__ds.CreateLayer(name_layer, srs, geom_type=ogr_type_geometry)
        if attribytes is not None:
            for att in attribytes:
                ogr_def = att.getOgrFieldDefn()
                if ogr_def is None:
                    continue
                layer.CreateField(ogr_def)
        return layer
    def close(self):
        if self.__ds is not None:
            self.__ds.Release()
            self.__ds=None

class OgrLayer:
    def __init__(self,size_cache=10000):
        self.__size_cache=size_cache
        self.__cache_ft=[]
        self.__counter_block=0
        self.__count_obj=0

    def createLayer(self,ds, name_layer, attribytes, srs_wkt, type_geometry=None):
        srs = osr.SpatialReference()
        if srs_wkt is None:
            srs.ImportFromEPSG(4326)
        else:
            srs.ImportFromWkt(srs_wkt)
        ogr_type_geometry = ogr.wkbUnknown
        if type_geometry == TypeGeometry.PolyLine or type_geometry == TypeGeometry.Vector:
            ogr_type_geometry = ogr.wkbLineString
        if type_geometry == TypeGeometry.Any:
            ogr_type_geometry = ogr.wkbUnknown
        if type_geometry == TypeGeometry.Polygon:
            ogr_type_geometry = ogr.wkbPolygon
        if type_geometry == TypeGeometry.Point:
            ogr_type_geometry = ogr.wkbPoint
        self.__layer = ds.CreateLayer(name_layer, srs, geom_type=ogr_type_geometry)
        if attribytes is not None:
            for att in attribytes:
                ogr_def = att.getOgrFieldDefn()
                if ogr_def is None:
                    continue
                self.__layer.CreateField(ogr_def)
        if self.__layer is not None:
            self.__featureDefn = self.__layer.GetLayerDefn()
            self.__cache_ft=[]
            return True
        return False
    @property
    def Count(self):
        return self.__count_obj
    def add(self,geo,id,tags:dict,type_object,style,type_osm):
        self.__count_obj=self.__count_obj+1
        ft=ogr.Feature(self.__featureDefn)
        ft.SetField('id',id)
        text_tags=json.dumps(tags ,ensure_ascii=False)
        ft.SetField('info_json',text_tags)
        ft.SetField('type_object', type_object)
        ft.SetField('mi_style', style)
        ft.SetField('osm_type', type_osm)
        if geo is not None:
            ft.SetGeometry(geo)
        geo=None
        self.__cache_ft.append(ft)
        if len(self.__cache_ft)>=self.__size_cache:
            self.__writeFeatures()
    def __writeFeatures(self):
        self.__counter_block=self.__counter_block+1
        self.__layer.StartTransaction()
        for ft in self.__cache_ft:
            self.__layer.CreateFeature(ft)
            ft=None
        self.__layer.CommitTransaction()
        self.__cache_ft.clear()
        print("Count block write:"+str(self.__counter_block))
        print()
    def close(self):
        if self.__layer is not None:
            if len(self.__cache_ft)>0:
                self.__writeFeatures()
            self.__layer=None
def existInTableGpkgExtention(db,tab_gpkg,name_tab):
    sql="Select * from "+tab_gpkg+" where table_name='"+name_tab+"'"
    rows=db.select(sql)
    if len(rows)==0:
        return False
    return True
def createSpatialView(db:DbTools,base_table,sql_view,name_view,sql_where):
    sql_create_view="create view "+name_view+" as "+sql_view+" from "+base_table
    if sql_where is not None:
        sql_create_view =sql_create_view +" where "+sql_where
    isOk,message_er=db.runSqlNoResult(sql_create_view)
    if not isOk:
        return False,sql_create_view+"\n"+message_er
    existIn=existInTableGpkgExtention(db,"gpkg_contents",name_view)
    if not existIn:
        sql=" select * from gpkg_contents where table_name='"+base_table+"'"
        rows_base_info=db.select(sql)
        row_tab=rows_base_info[0]
        sql_insert_contents='Insert into gpkg_contents ("table_name","data_type","identifier","min_x","min_y","max_x","max_y","srs_id")'
        sql_insert_contents=sql_insert_contents+" values(?,?,?,?,?,?,?,?)"
        value_insert=(name_view,"features",name_view,row_tab[5],row_tab[6],row_tab[6],row_tab[8],row_tab[9])
        db.insert(sql_insert_contents,value_insert)
    existIn = existInTableGpkgExtention(db,"gpkg_geometry_columns", name_view)
    if not existIn:
        sql = " select * from gpkg_geometry_columns where table_name='" + base_table + "'"
        rows_base_info = db.select(sql)
        row_tab = rows_base_info[0]
        sql_insert='Insert into gpkg_geometry_columns ("table_name","column_name","geometry_type_name","srs_id","z","m") '
        sql_insert =sql_insert+" values(?,?,?,?,?,?)"
        values=(name_view,row_tab[1],row_tab[2],row_tab[3],0,0)
        db.insert(sql_insert,values)
    return True,None
def prepareResultImport(db,property_import):
    name_layer=property_import.name_layer
    list_sql=property_import.sql_view
    if isinstance(list_sql,dict):
        isOk,message_err=createSpatialView(db,property_import.name_layer,list_sql['sql'],list_sql['name_view'],list_sql['where'])
        if isOk:
            return [list_sql['name_view']]
        return None
    list_view=[]
    for item_sql in list_sql:
        isOk,message_err=createSpatialView(db, property_import.name_layer, item_sql['sql'], item_sql['name_view'], item_sql['where'])
        if isOk:
            list_view.append(item_sql['name_view'])
    if len(list_view)==0:
        return None
    return list_view
def createSpatialViewOne(db,name_view,base_tab,fields_select,where):
    sql='DROP VIEW IF EXISTS '+name_view
    db.runSqlNoResult(sql)
    sql_base="Select "+fields_select
    '''
    sql_base=sql_base+" from "+base_tab
    if where is not None:
        sql_base=sql_base+" where "+where
    #sql_create="create view "+name_view+" as "+sql_base
    '''
    is_ok,message_error=createSpatialView(db,base_tab,sql_base,name_view,where)
    return is_ok,message_error






