import binascii

import osmium as o
from PySide2.QtCore import QObject, Signal
from osgeo import ogr

from ru_axioma_gis_loadOsm.ui.tools.add_helper.handlers_ogr import Attribute, DbOgr, ogrLineStringToPolygon

geojsonfab = o.geom.WKBFactory()
def createOgrGeometry(str_geo):
    byte_geo_obj = binascii.unhexlify(str_geo)
    # poly = wkblib.loads(wkb, hex=True)
    ogr_geo = ogr.CreateGeometryFromWkb(byte_geo_obj)
    return ogr_geo

class GeoOgrDbWriter(o.SimpleHandler,QObject):
    change_progress=Signal(int)
    def __init__(self,path_file,name_layer,property_import,geo_filter=None,size_cache=10000):
        #o.SimpleHandler.__init__()
        QObject.__init__(self)
        self.__max_count=None
        self.__counter=0
        self.__all_count_object=0
        self.__size_cache=size_cache
        self.__geo_filter=geo_filter
        self.__dbOgr=DbOgr(path_file)
        self.__dbOgr.createSpatial()
        self.__property_ipmort=property_import
        self.__step_progress=100
        self.__layer=None
        self.__createLayer(name_layer)
        self.__all_tags={}
        self.__isCancel=False
        self.existRelation=False
    def setStepProgress(self,step):
        self.__step_progress=step
    def setIsCancel(self):
        self.__isCancel=True


    def __createLayer(self,name_layer):
        attributes = []
        attributes.append(Attribute('id', "int64"))
        attributes.append(Attribute('info_json', "string", 8000))
        attributes.append(Attribute('type_object', "int"))
        attributes.append(Attribute('mi_style', "string", 254))
        attributes.append(Attribute('osm_type', "string", 12))
        self.__layer=self.__dbOgr.createLayer(name_layer,attributes,None)
    @property
    def CountObjectImport(self):

        return self.__all_count_object
    @property
    def allTags(self):
        return self.__all_tags

    def node(self, o):
        if not self.__property_ipmort.point.enable:
            return
        if o.tags:

            self.__counterTry()
            str_byte_geo_obj=geojsonfab.create_point(o)
            ogr_geo=createOgrGeometry(str_byte_geo_obj)
            if not self.__inGeoFilter(ogr_geo):
                return
            self.__addUnicalkey(o.tags)
            id = o.id
            self.__layer.add(ogr_geo, id, dict(o.tags),0,self.__property_ipmort.point.style,"node")



    def way(self, o):

        if o.tags:
            if not self.__property_ipmort.way.enable:
                return
            self.__counterTry()
        #if o.tags and not o.is_closed():
            #if (not o.is_closed()) and (not self.__property_ipmort.way.polygon)
            str_byte_geo_obj=None
            try:
                str_byte_geo_obj = geojsonfab.create_linestring(o)
            except Exception as ex:
                print(ex)
            if str_byte_geo_obj is None:
                return
            ogr_geo = createOgrGeometry(str_byte_geo_obj)
            if ogr_geo.GetGeometryName()!='LINESTRING':
                jkl=0
            id = o.id
            if not self.__inGeoFilter(ogr_geo):
                return
            self.__addUnicalkey(o.tags)

            if o.is_closed() and self.__property_ipmort.way.polygon:
                geo_polygon=ogrLineStringToPolygon(ogr_geo)
                ogr_geo=None
                self.__layer.add(geo_polygon, id, dict(o.tags), 3,self.__property_ipmort.area.style,'way')
            else:
                self.__layer.add(ogr_geo,id,dict(o.tags),1,self.__property_ipmort.way.style,"way")


    def area(self, o):

        if o.tags and self.__property_ipmort.area.enable:

            self.__counterTry()
            str_byte_geo_obj=geojsonfab.create_multipolygon(o)

            ogr_geo = createOgrGeometry(str_byte_geo_obj)
            id=o.id
            if not self.__inGeoFilter(ogr_geo):
                return
            self.__addUnicalkey(o.tags)
            self.__layer.add(ogr_geo, id, dict(o.tags),2,self.__property_ipmort.area.style,"area")
            #self.__addUnicalkey(o.tags)
            #self.print_object(geojsonfab.create_multipolygon(o), o.tags)

    def relation(self, o):
        if o.tags:
            self.existRelation=True
            if self.__property_ipmort.relation.enable:
                self.__layer.add(None, o.id, dict(o.tags), 1, self.__property_ipmort.area.style, "relation")
    def __inGeoFilter(self,inp_geometry):
        if inp_geometry is None:
            return False
        if self.__geo_filter is None:
            return True
        if self.__geo_filter.Intersects(inp_geometry):
            return True
        if self.__geo_filter.Contains(inp_geometry):
            return True
    def __counterTry(self):
        self.__counter = self.__counter + 1
        '''
        if self.__max_count is None:
            return
        if self.__counter>self.__max_count:
            raise  ValueError("I have raised an Exception")
        '''
        if (self.__counter % self.__step_progress)==0:
            self.change_progress.emit(self.__counter)
        if self.__isCancel:
            raise ValueError("I have raised an Exception")
    def __addUnicalkey(self,tags):
        dict_tags=dict(tags)
        for key in dict_tags.keys():
            if key in self.__all_tags.keys():
                continue
            len_field=len(dict_tags[key])
            self.__all_tags[key]=len_field
    def finish(self):
        if self.__layer is not None:
            self.__all_count_object=self.__layer.Count
            self.__layer.close()
            self.__layer=None
        if self.__dbOgr is not None:
            self.__dbOgr.close()
            self.__dbOgr=None
