import os
from pathlib import Path

from shapely import LineString, multilinestrings, line_merge

from ru_axioma_gis_loadOsm.ui.tools.add_helper.db.dbTools import DbTools
from ru_axioma_gis_loadOsm.ui.tools.add_helper.relation_tools import getMembersRelation

def listWayToDict(list_way):
    dict_way={}
    for way in list_way:
        dict_way[way['id']]=way['nodes']
    return dict_way

class RelationDb:
    __name_relation='relation'
    __name_way='way'
    __name_point='points'
    __name_link='link_way_to_points'
    def __init__(self,path_db,path_osm,add_exist=True):
        self.__path_db=path_db
        self.__osm_file=path_osm
        self.__db=None
        self.__location_cache=None
        if not add_exist:
            if Path(path_db).exists():
                os.remove(path_db)
            self.__db=DbTools(path_db)
            self.__createDb()
        if self.__db is None:
            self.__db = DbTools(path_db)
    def createCache(self,path_cache=None):
        self.__path_cache=path_cache
        if path_cache is None:
            import osmium as o
            name_file=Path(self.__osm_file).stem
            path_file_cache=Path(self.__osm_file).parent.joinpath(name_file+".cache")
            str_path_file_cache=str(path_file_cache)
            if path_file_cache.exists():
                os.remove(str_path_file_cache)
            self.__path_cache=str_path_file_cache
            reader = o.io.Reader(self.__osm_file, o.osm.osm_entity_bits.NODE)

            idx = o.index.create_map("sparse_file_array," + self.__path_cache)

            lh = o.NodeLocationsForWays(idx)

            o.apply(reader, lh)

            reader.close()
            self.__location_cache=idx
    def getGeometryRelation(self,id_relation,combine_result=True):
        sql_relation="select id_osm from "+self.__name_relation+" where id_osm="+str(id_relation)
        rows_relations=self.__db.select(sql_relation)
        if len(rows_relations)==0:
            ''' такой relation отсутствует'''
            self.__getPointsRelation(id_relation)
        geometrys=self.__getGeometry(id_relation)
        if geometrys is None:
            return None
        if len(geometrys)==0:
            return geometrys[0]
        if combine_result:
            geo_multi_linestring = multilinestrings(geometrys)
            return line_merge(geo_multi_linestring)
        return geometrys
    def __getGeometry(self,id_relation):
        ''' выбираем все дочерние элементы'''
        sql="select id_osm from way where parent_osm="+str(id_relation)
        rows_way=self.__db.select(sql)
        collection_ways=[]
        for way in rows_way:
            id_way=way[0]
            geo=self.__geometryWay(id_way)
            if geo is None:
                continue
            collection_ways.append(geo)
        return collection_ways
    def __geometryWay(self,id_way):
        sql_link="select * from link_way_to_points where id_parent="+str(id_way)
        sql_main="select pnt.x,pnt.y from ("+sql_link+") link,points as pnt where link.id_point==pnt.id_osm"
        row_points=self.__db.select(sql_main)
        if len(row_points)==0:
            return None
        points=[]
        for row in row_points:
            points.append((row[0],row[1]))
        geo=LineString(points)
        return geo
    def __getPointsRelation(self,id_relation):
        ''' get id objects in relation'''
        idx_realation=getMembersRelation(self.__osm_file,id_relation)
        if len(idx_realation.relations)==0:
            return
        sql_insert="insert into "+self.__name_relation+" (id_osm) VALUES("+str(id_relation)+")"
        self.__db.runSqlNoResult(sql_insert)
        list_way = idx_realation.relations[0]['children']
        if len(list_way)==0:
            return
        no_exist_obj=self.__getChildrenObject(list_way,self.__name_way)
        if no_exist_obj is not None:
            ''' ситаем что если все way (дочерние объекты существуют) , и точки в базе'''
            #return

            idx_points=getMembersRelation(self.__osm_file,no_exist_obj)
            for children_obj in idx_points.ways:
                self.__saveLinkWayPoints(children_obj)
                nodes=children_obj['nodes']
                list_no_exist=self.__getChildrenObject(nodes,self.__name_point)
                if list_no_exist is None or len(list_no_exist)==0:
                    continue
                #idx_point_values=getMembersRelation(self.__osm_file,list_no_exist)
                idx_point_values=self.__getPointsValue(list_no_exist)
                if self.__location_cache is None:
                    self.__savePointsToDb(idx_point_values.nodes)
                else:
                    self.__savePointsToDb(idx_point_values)
        values_way=[]
        for way in list_way:
            values_way.append((way,id_relation))
        sql_insert="insert into "+self.__name_way+" (id_osm,parent_osm) values(?,?)"
        self.__db.insertMany(sql_insert,values_way)
        return
    def __getPointsValue(self,list_node):
        if self.__location_cache is None:
            return getMembersRelation(self.__osm_file,list_node)
        nodes_info={}
        for node_id in list_node:
            n=self.__location_cache.get(node_id)
            nodes_info[node_id] = {'id': node_id, 'x': n.lon, 'y': n.lat}
        return nodes_info

    def __savePointsToDb(self,list_nodes):
        values=[]
        for key in list_nodes.keys():
            node=list_nodes[key]
            values.append((key,node['x'],node['y']))
        sql_insert="insert into "+self.__name_point+" (id_osm,x,y) values(?,?,?)"
        self.__db.insertMany(sql_insert,values)
        return
    def __saveLinkWayPoints(self,obj_way):
        id_way=obj_way['id']
        nodes=obj_way['nodes']
        values=[]
        for node in nodes:
            values.append((id_way,node))
        sql_insert="insert into "+self.__name_link+" (id_parent,id_point) values(?,?)"
        self.__db.insertMany(sql_insert,values)
        return
    def __getChildrenObject(self,list_children,name_tab):
        no_exist_children=[]
        sql_exist="select id_osm from "+name_tab+" where id_osm="
        for ch_obj in list_children:
            rows=self.__db.select(sql_exist+str(ch_obj))
            if len(rows)==0:
                no_exist_children.append(ch_obj)
            #else:
                #print("way from db :"+str(ch_obj))
        if len(no_exist_children)==0:
            ''' все уже в базе '''
            return None
        return no_exist_children





    def __createDb(self):
        sql='CREATE TABLE [relation]([id_osm] INT64 PRIMARY KEY NOT NULL UNIQUE)'
        self.__db.runSqlNoResult(sql)
        sql='CREATE TABLE [way](   [id_osm] INT64 , [parent_osm] INT64 )'
        self.__db.runSqlNoResult(sql)
        sql='CREATE INDEX [indx_base_id_way] ON [way]([id_osm])'
        self.__db.runSqlNoResult(sql)
        sql = 'CREATE INDEX [index_parent] ON [way]([parent_osm])'
        self.__db.runSqlNoResult(sql)
        sql = 'CREATE TABLE [link_way_to_points](   [id_parent] INT64 NOT NULL REFERENCES [way]([id_osm]),   [id_point] INT64)'
        self.__db.runSqlNoResult(sql)
        sql = 'CREATE INDEX [indx_link_points] ON [link_way_to_points]([id_point])'
        self.__db.runSqlNoResult(sql)
        sql='CREATE TABLE [points](  [id_osm] INT64 PRIMARY KEY NOT NULL UNIQUE,   [x] FLOAT NOT NULL,   [y] FLOAT NOT NULL)'
        self.__db.runSqlNoResult(sql)

    def close(self,commit=True):
        if self.__location_cache is not None:

            self.__location_cache=None
            try:
                os.remove(self.__path_cache)
            except:
                pass
        self.__db.close(commit)