import os
from pathlib import Path
from typing import Union

from PySide2.QtCore import QObject, Signal
import osmium as o

from ru_axioma_gis_loadOsm.ui.tools.add_helper.db.dbTools import DbTools
from ru_axioma_gis_loadOsm.ui.tools.add_helper.ogrWriteHandler import GeoOgrDbWriter


class LoaderOsm(QObject):
    update_progess=Signal(int)
    def __init__(self):
        super().__init__()
        self.__handler=None
        self.__isCancel=False
        self.__progress_count = 0
        self.__count_obj_load=0
    def setParams(self,path_source,path_dest,name_layer,property_import,geo_filter):
        self.__path_source=path_source
        self.__path_dest=path_dest
        self.__name_layer=name_layer
        self.__property_import=property_import
        self.__geo_filter=geo_filter
    def run(self):
        self.__progress_count=0
        #self.run_importNew(self.__path_source,self.__path_dest,self.__name_layer,self.__property_import,self.__geo_filter)
        self.run_import(self.__path_source, self.__path_dest, self.__name_layer, self.__property_import,
                           self.__geo_filter)
    def run_import(self,path_source,path_dest,name_layer,property_import,geo_filter=None):
        if Path(path_dest).exists():
            os.remove(path_dest)
        if property_import.base_tag!='*':
            filter_building = o.filter.KeyFilter(property_import.base_tag)
        else:
            filter_building = o.filter.EmptyTagFilter()

        self.__handler = GeoOgrDbWriter(path_dest, name_layer,property_import, geo_filter)
        self.__handler.change_progress.connect(self.__change_progress_bar)
        try:
            self.__handler.apply_file(path_source, filters=[filter_building],idx='sparse_file_array')
        except Exception as ex:
            jkl = 0
        self.__handler.finish()
        self.__handler.change_progress.disconnect(self.__change_progress_bar)
        self.__count_obj_load = self.__handler.CountObjectImport
        self.update_progess.emit(100)
        db=DbTools(path_dest)
        sql_crate = "CREATE TABLE [tags](  [id] INTEGER PRIMARY KEY AUTOINCREMENT UNIQUE,  [name] TEXT,  [size_field] INTEGER)"
        db.runSqlNoResult(sql_crate)
        sql_insert = "Insert into tags (name,size_field) values(?,?)"
        rows_tags=[]
        tags_info=self.__handler.allTags
        for key in tags_info.keys():
            rows_tags.append((key,tags_info[key]))
        db.insertManyTransaction(sql_insert,rows_tags)
        db.close(True)
        geo_filter=None
    def run_importNew(self,path_source,path_dest,name_layer,property_import,geo_filter=None):
        if Path(path_dest).exists():
            os.remove(path_dest)
        filter_nodes=None
        if property_import.point.enable:
            filter_nodes=o.osm.NODE
        if property_import.way.enable:
            if filter_nodes is None:
                filter_nodes=o.osm.Way
            else:

                filter_nodes=o.osm.WAY | filter_nodes
        if property_import.area.enable:
            if filter_nodes is None:
                filter_nodes=o.osm.AREA
            else:
                filter_nodes=filter_nodes | o.osm.AREA
        self.__handler = GeoOgrDbWriter(path_dest, name_layer, property_import, geo_filter)
        self.__handler.change_progress.connect(self.__change_progress_bar)
        filter_nodes = filter_nodes |  o.osm.RELATION
        filter_building = o.filter.KeyFilter(property_import.base_tag).enable_for(filter_nodes)
        filter_building = o.filter.KeyFilter("boundary")
        #filter_nodes.enable_for(filter_nodes)
        #for obj in o.FileProcessor(path_source,filter_nodes).with_filter(filter_building):
        file_processing=o.FileProcessor(path_source)
        #file_processing.with_filter(filter_building)
        #file_processing.with_filter(o.filter.KeyFilter("boundary"))
        for obj in file_processing:

            if self.__isCancel:
                break
            if obj.is_node():
                self.__handler.node(obj)
            elif obj.is_way():
                self.__handler.way(obj)
            elif obj.is_area():
                self.__handler.area(obj)
            elif obj.is_relation():
                self.__handler.relation(obj)

        self.__handler.finish()
    @property
    def countObject(self):
        return self.__count_obj_load
    def setIsCancel(self):
        self.__isCancel=True
        #self.__handler.setIsCancel()
    def __change_progress_bar(self,int):
        self.__progress_count=self.__progress_count+1
        if self.__progress_count>=100:
            self.__progress_count=0
        self.update_progess.emit(self.__progress_count)
        if self.__isCancel:
            self.__handler.setIsCancel()


