import os

import axipy
from PySide2.QtCore import QFile
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QTabWidget
from axipy import Notifications, show_dialog, DlgIcon, DlgButtons, provider_manager

from ru_axioma_gis_loadOsm.ui.tools.add_helper.db.dbTools import DbTools, isDefaultGpkgName
from ru_axioma_gis_loadOsm.ui.tools.add_helper.handlers_ogr import createSpatialViewOne


class DlgCreateView(QWidget):
    __name_widget = "Создание представлений (view)"
    def __init__(self,parent=None):
        super().__init__(parent)
        self.__gpkg_is_osm=False
        self.__ui=self.load_ui("widget_create_view.ui",parent)
        if isinstance(parent, QTabWidget):
            self.__isTab = True
            count_tab = parent.count()
            self.__idTab = count_tab - 1
            parent.setTabText(self.__idTab, self.__name_widget)
        self.__ui.tb_open_gpkg.clicked.connect(self.__open_gpkg)
        self.__ui.pb_run.clicked.connect(self.__run)
    def load_ui(self,name_resource,parent):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__),name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        ui  = loader.load(ui_file,parent)
        ui_file.close()
        return ui
    def __open_gpkg(self):
        title = 'Файл Gpkg'
        filter = 'Gpkg  (*.gpkg)'
        folder = None
        selected_filter = 'Gpkg  (*.gpkg)'
        res = axipy.open_files_dialog(filter, title, folder, selected_filter)
        if res is None:
            return
        self.__gpkg_is_osm = False
        str_path_open = str(res[0])
        self.__ui.ln_source_gpkg.setText(str_path_open)
        self.__ui.ln_source_gpkg.setCursorPosition(0)
        self.__ui.cmb_tables.clear()
        self.__try_is_gpkg_osm(str_path_open)
        self.__ui.pb_run.setEnabled(self.__gpkg_is_osm)



    def __run(self):
        path_db=self.__ui.ln_source_gpkg.text()
        name_view=self.__ui.ln_name_view.text().strip()
        if len(name_view)==0:
            Notifications.push('Создание представлений (view)', 'Имя представления не определено', Notifications.Critical)
            return
        list_name_fields = self.__ui.tx_base_sql.toPlainText().strip().replace("\n",'')
        if len(list_name_fields)==0:
            Notifications.push('Создание представлений (view)', 'Перечень полей пуст', Notifications.Critical)
            return
        try:
            db = DbTools(path_db)
            isExistView=db.existTable(name_view,'view')
            if isExistView:
                res = show_dialog('Представление с именем существует. Заменить?', 'Создание представлений',
                                  icon=DlgIcon.QUESTION,
                                  default_button=DlgButtons.CANCEL,
                                  buttons=DlgButtons.YES_NO_CANCEL)
                if res!= DlgButtons.YES:
                #Notifications.push('Создание представлений (view)', 'Представление с именем:'+name_view+" ceotcndetn", Notifications.Critical)
                    return
            is_not_geom=True
            if list_name_fields.find('geom')>=0:
                is_not_geom=False
            if is_not_geom:
                list_name_fields=list_name_fields+",geom"
            str_where=self.__ui.ln_sql_where.text().strip()
            if len(str_where)==0:
                str_where=None
            base_table=self.__ui.cmb_tables.currentText()
            is_ok,message=createSpatialViewOne(db,name_view,base_table,list_name_fields,str_where)
            if not is_ok:
                Notifications.push('Создание представлений (view)', message, Notifications.Critical)
                return
            db.close(True)
            db=None
        except Exception as ex:
            pass
        finally:
            if db is not None:
                db.close()
        definition = {'src': path_db,
                      'dataobject': name_view}
        table_view = provider_manager.open(definition)
        axipy.view_manager.create_tableview(table_view)
    def __try_is_gpkg_osm(self,path_gpkg):
        db_temp=DbTools(path_gpkg)
        self.__gpkg_is_osm=db_temp.existTable("tags")
        if self.__gpkg_is_osm:
            sql = "SELECT name FROM sqlite_master WHERE type='table'"
            rows =  db_temp.select(sql)
            for row in rows:
                name_tab=row[0]
                isDef_gpkg=isDefaultGpkgName(name_tab)
                if isDef_gpkg:
                    continue
                self.__ui.cmb_tables.addItem(name_tab)
        db_temp.close(False)
    @property
    def widget(self):
        return self.__ui
