import os
from pathlib import Path

import axipy
from PySide2.QtCore import QFile
from PySide2.QtGui import QIcon
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QTabWidget
from axipy import provider_manager, Layer

from ru_axioma_gis_loadOsm.ui.tools.add_helper.Observers import factoryObserverSelectPolygonsNew
from ru_axioma_gis_loadOsm.ui.tools.add_helper.db.dbTools import DbTools
from ru_axioma_gis_loadOsm.ui.tools.add_helper.handlers_ogr import prepareResultImport
from ru_axioma_gis_loadOsm.ui.tools.add_helper.helper_select_geometry import getSelectionPolygon
from ru_axioma_gis_loadOsm.ui.tools.add_helper.property_inport import loadAllPropery
from ru_axioma_gis_loadOsm.ui.tools.loadOsm import LoaderOsm


class DlgLoadOsm(QWidget):
    __name_widget="Импорт Osm (pbf)"
    def __init__(self,parent_object,parent=None):
        super().__init__(parent)
        self.__observer=factoryObserverSelectPolygonsNew("SelectPolygons")
        self.__parent=parent
        self.__run_task=None
        self.__cls_run=None
        self.__curent_property_import=None
        self.__parent_plugin=parent_object
        self.__ui=self.load_ui("widget_loadOsm2.ui",parent)
        self.__observer.changed.connect(self.__change_observer)
        curent_path = str(Path(__file__).parent)
        path_icon = os.path.join(curent_path, "icons")
        path_open_png = os.path.join(path_icon, "open-32.png")
        path_saveas_png = os.path.join(path_icon, "saveas.png")

        self.__ui.tb_open.setIcon(QIcon(path_open_png))
        self.__ui.tb_saveas.setIcon(QIcon(path_saveas_png))
        if isinstance(parent, QTabWidget):
            self.__isTab = True
            count_tab = parent.count()
            self.__idTab = count_tab - 1
            self.__parent.setTabText(self.__idTab, self.__name_widget)
        self.__ui.tb_open.clicked.connect(self.__select_pbf)
        self.__ui.tb_saveas.clicked.connect(self.__select_gpkg)
        self.__ui.cmb_filter.currentIndexChanged.connect(self.__change_filter)
        self.__ui.pb_run.clicked.connect(self.__run)
        self.__ui.pb_cancel.clicked.connect(self.__cancel)
        self.__init_filter()

    def load_ui(self,name_resource,parent):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__),name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        ui  = loader.load(ui_file,parent)
        ui_file.close()
        return ui
    def __init_filter(self):
        '''
        self.__ui.cmb_filter.addItem("building")
        self.__ui.cmb_filter.addItem("boundary")
        self.__ui.cmb_filter.setCurrentIndex(0)
        '''
        self.__all_propery=loadAllPropery(self.__parent_plugin.get_plugin_data_dir())
        if self.__all_propery is None:
            return
        for key in self.__all_propery.keys():
            self.__ui.cmb_filter.addItem(key)
    def __change_observer(self):
        self.__ui.ch_use_spatial_filter.setEnabled(self.__observer.value)

    def __select_pbf(self):
        title = 'Файл Osm'
        filter = 'Osm  (*.pbf)'
        folder = None
        selected_filter = 'Osm  (*.pbf)'
        res = axipy.open_files_dialog(filter, title, folder, selected_filter)
        if res is None:
            return
        str_path_open=str(res[0])
        self.__ui.ln_source_osm.setText(str_path_open)
        self.__ui.ln_source_osm.setCursorPosition(0)
        self.__setDefaultOutName()
        self.__enableRun()
    def __select_gpkg(self):
        title = 'Файлы Gpkg'
        filter = 'Gpkg (*.gpkg)'
        folder = None
        selected_filter = 'Gpkg (*.gpkg)'
        file_name=None
        if len(self.__ui.ln_out_gpkg.text())>0:
            file_name=self.__ui.ln_out_gpkg.text()
        res=axipy.save_file_dialog(filter,title,folder,file_name,selected_filter)
        if res is None:
            return
        str_path_open = str(res)
        self.__ui.ln_out_gpkg.setText(str_path_open)
        self.__ui.ln_out_gpkg.setCursorPosition(0)
        self.__enableRun()
    def __change_filter(self):
        curent_filter=self.__ui.cmb_filter.currentText()
        select_file=self.__ui.ln_source_osm.text()
        if len(select_file)>0:
            name_file=Path(self.__ui.ln_source_osm.text()).stem
            name_file=name_file.replace(".","_")
            file_out=name_file+"_"+curent_filter+".gpkg"
            self.__ui.ln_out_gpkg.setText(file_out)
            self.__ui.ln_out_gpkg.setCursorPosition(0)
    def __setDefaultOutName(self):
        curent_filter = self.__ui.cmb_filter.currentText()
        name_file = Path(self.__ui.ln_source_osm.text()).stem
        name_file = name_file.replace(".", "_")
        file_out = name_file + "_" + curent_filter + ".gpkg"
        path_out=Path(self.__ui.ln_source_osm.text()).parent.joinpath(file_out)
        self.__ui.ln_out_gpkg.setText(str(path_out))
        self.__ui.ln_out_gpkg.setCursorPosition(0)
    def __enableRun(self):
        if self.__all_propery is None:
            self.__ui.pb_run.setEnabled(False)
            return
        path_source=self.__ui.ln_source_osm.text()
        path_dest=self.__ui.ln_out_gpkg.text()
        if Path(path_source).exists() and len(path_dest)>0:
            self.__ui.pb_run.setEnabled(True)
        else:
            self.__ui.pb_run.setEnabled(False)
    def __run(self):
        self.__ui.pb_run.setEnabled(False)
        #self.__ui.pb_cancel.setText("Прервать")
        self.__ui.pb_cancel.setEnabled(True)
        osmfile=self.__ui.ln_source_osm.text()
        path_db_osm=self.__ui.ln_out_gpkg.text()
        select_filter=self.__ui.cmb_filter.currentText()
        propert_import=self.__all_propery[select_filter]
        propert_import.point.enable=self.__ui.ch_point.isChecked()
        propert_import.way.enable=self.__ui.grb_way.isChecked()
        propert_import.relation.enable=self.__ui.cb_relation.isChecked()
        if propert_import.way:
            propert_import.way.polygon=self.__ui.ch_close_poly.isChecked()
        else:
            propert_import.way.polygon=False
        propert_import.area.enable=self.__ui.ch_polygons.isChecked()
        geo_filter=None
        if self.__observer.value:
            geo_filter=getSelectionPolygon()
        name_layer=propert_import.name_layer
        self.__curent_property_import=propert_import
        if Path(path_db_osm).exists():
            os.remove(path_db_osm)
        self.__cls_run = LoaderOsm()
        self.__cls_run.setParams(osmfile,path_db_osm,name_layer,propert_import,geo_filter)
        self.__cls_run.update_progess.connect(self.__update_progess)

        def run_process(t: axipy.Task):
            self.__cls_run.run()

        self.__run_task = axipy.Task(run_process)
        self.__run_task.finished.connect(self.__finish_task)
        self.__run_task.start()
    def __finish_task(self):
        self.__ui.pb_run.setEnabled(True)
        #self.__ui.pb_cancel.setText("Закрыть")
        self.__ui.pb_cancel.setEnabled(False)
        count_obj=self.__cls_run.countObject
        self.__cls_run.update_progess.disconnect(self.__update_progess)
        self.__run_task=None

        self.__cls_runner=None
        if count_obj>0 and self.__curent_property_import is not None:
            db = DbTools(self.__ui.ln_out_gpkg.text())
            list_view=prepareResultImport(db,self.__curent_property_import)

            db.close(True)
            self.__curent_property_import=None
            if self.__ui.chb_view_layer.isChecked() and list_view is not None:
                definition = {'src': self.__ui.ln_out_gpkg.text(),
                              'dataobject': list_view[0]}
                table_view=provider_manager.open(definition)
                axipy.mainwindow.add_layer_interactive(Layer.create(table_view))
            return


    def __update_progess(self,value):
        #print(value)
        self.__ui.progressBar.setValue(value)
    def __cancel(self):
        if self.__cls_run is not None:
            self.__cls_run.setIsCancel()

    @property
    def widget(self):
        return self.__ui
